# Access Token Auth
The Access Token Auth module enables the generation of access tokens for authenticating requests within Drupal.

Other modules that this was based on:
- [REST API Access Token](https://www.drupal.org/project/rest_api_access_token)
- [Simple OAuth (OAuth2) & OpenID Connect](https://www.drupal.org/project/simple_oauth)

## Features
The Access Token Auth module focuses solely on the creation of access tokens, making it a straightforward solution. However, it's important to consider permissions because the generated token will authenticate requests as the user who generated the token, which is the currently logged-in user.

### Configuration
The module offers two types of settings for the generated access tokens: `Time-based` and `One-time use`.

If the user modifies the settings while tokens have already been generated, the existing tokens will adapt to the current settings. This means that regardless of the original configuration, the module automatically adjusts the tokens. For example, if a token was generated with the `Time-based` option and has already been used but remains valid within the time window, changing the settings to `One-time` will mark the token as used, as it was already used. The same behavior applies in the opposite scenario.

#### Time-based
This is the default option, which sets a Time To Live (TTL) in seconds for the generated access tokens. The tokens remain valid for a specified period of time. By default, the TTL value is set to `180` seconds.

#### One-time use
This option allows tokens to be used only once.

### Authentication
To authenticate using the generated token, you have two options: `headers` or `parameters`.

Regardless of your choice, you only need to include the `X-ACCESS-AUTH-TOKEN` variable with the token value.

## Usage
Using the Access Token Auth module is straightforward. Follow these steps:

### Generate token
1. Enable the module.
2. Go to `/admin/config/services/access-token-auth-settings`.
3. Configure the settings according to your preferences and save the changes.
4. Press the `Generate Token` button.

### Authenticate with access token
1. In your preferred client, pass the token value using the `X-ACCESS-AUTH-TOKEN` variable name.
   - You can pass in via headers or query paramaters.
2. That's it!

## How to use in code
The module provides the `Token Manager Factory` service, which is the main service used to load and manage token managers for access and token validation. Additionally, there are other related services available.

### Token Manager Factory
The `access_token_auth.token_manager_factory` service is responsible for loading the default or selected token manager service. The returned service implements the `TokenManagerInterface` interface.

#### Token Manager Interface
The `TokenManagerInterface` interface defines the methods required to implement a custom `Token Manager`. This allows for flexibility and scalability in case the default token service does not meet specific requirements. By implementing a custom `Token Manager`, you can create, validate, and authenticate access tokens. The default `Token Manager` is already provided, but custom implementations can be used to fulfill specific use cases.

### Default Token Manager
The default token manager service (`access_token_auth.default_token_manager`) is responsible for creating, validating, and authenticating access tokens. It associates access tokens with the user who generates them and stores them in the database for validation. In general, it is recommended to use the `Token Manager Factory` service instead of directly accessing the default token manager service in your code.

### Other services
- **Token Generator**: The token generator service (`access_token_auth.token_generator`) implements the `TokenGeneratorInterface` and is responsible for creating unique token values.

### Token Manager Service Tag
The `access_token_auth.token_manager` service tag allows you to discover and use custom `Token Manager` services in your own site.

### How to create your own custom Token Manager
To create your own custom `Token Manager`, follow these steps:

1. Create a new service.
2. Implement the `TokenManagerInterface` interface in your service.
3. Tag your service with the custom `access_token_auth.token_manager` tag.
4. That's it! Your custom `Token Manager` is ready to be used.

### How to use your custom Token Manager
To use your custom Token Manager, follow these steps:

1. Go to `/admin/config/services/access-token-auth-settings`.
2. Expand the `Advanced Settings` tab.
3. Select your custom `Token Manager`.
4. Save the configuration.
5. That's it! Your custom `Token Manager` will now be used for the specified actions.
