<?php

namespace Drupal\access_token_auth\Model;

/**
 * Token model class.
 */
class Token {

  /**
   * The token database table name to register tokens.
   */
  const TABLE_NAME = 'access_token_auth';

  /**
   * The default time to live of a token in seconds (3 minutes).
   */
  const DEFAULT_TTL = 180;

  /**
   * The default token mode use.
   */
  const DEFAULT_MODE = 'time_based';

  /**
   * The token.
   *
   * @var string
   */
  protected string $token;

  /**
   * The user ID.
   *
   * @var int
   */
  protected int $userId;

  /**
   * The token's expiration date.
   *
   * @var int
   */
  protected int $expireAt;

  /**
   * Flag to indicate whether the token has been used or not.
   *
   * @var bool
   */
  protected bool $usedStatus;

  /**
   * Token class constructor.
   *
   * @param string $token
   *   The token.
   * @param int $user_id
   *   The user id.
   * @param int $expire_at
   *   The expiration date timestamp.
   * @param bool $used_status
   *   Whether the token has been used or not.
   */
  public function __construct(string $token, int $user_id, int $expire_at, bool $used_status = FALSE) {
    $this->token = $token;
    $this->userId = $user_id;
    $this->expireAt = $expire_at;
    $this->usedStatus = $used_status;
  }

  /**
   * Get token.
   *
   * @return string
   *   Return the token value.
   */
  public function getToken(): string {
    return $this->token;
  }

  /**
   * Get user id.
   *
   * @return int
   *   Return user id.
   */
  public function getUserId(): int {
    return $this->userId;
  }

  /**
   * Get the token's expiration date timestamp.
   *
   * @return int
   *   The expiration date timestamp
   */
  public function getExpireAt(): int {
    return $this->expireAt;
  }

  /**
   * Get the token's expiration date in a date time format.
   *
   * @return \DateTime
   *   The expiration date.
   */
  public function getExpireAtDate(): \DateTime {
    return (new \DateTime())->setTimestamp($this->expireAt);
  }

  /**
   * Get the token's used status.
   *
   * @return bool
   *   Flag to indicate whether the token is used or not.
   */
  public function getUsedStatus(): bool {
    return $this->usedStatus;
  }

}
